/*
 * common.h
 */

#define NR_GROUPS	10
#define THREADS_PER_GRP	192
#define MD5_BLK_BYTES	64
#define DATA_BYTES	(2 * MD5_BLK_BYTES) // 2 blocks
#define SINE_BYTES	(64 * 4) // 64 uint values
#define IHV_BYTES	(2 * 16) // 2 IHVs
#define PATH_RES_BYTES	80
#define OUTPUT_BYTES	(NR_GROUPS * THREADS_PER_GRP * PATH_RES_BYTES)

/*
 * Each thread computes a path in the 64-bit search space, and saves the
 * result in a binary structure of PATH_RES_BYTES bytes:
 *   4 x uint32   Original x values
 *   4 x uint32   Original y values
 *   4 x uint32   Computed x values after a certain number of iterations
 *   4 x uint32   Computed y values after a certain number of iterations
 *   4 x uint32   Number of iterations
 */

typedef struct	s_ihv
{
    uint32_t	a, b, c, d;
}		t_ihv;

typedef struct	s_node
{
    uint32_t	x, y;
}		t_node;

uint32_t swapu32(uint32_t i);
void read_node(t_node *n, const char *s);
void read_blocks_ihvs(unsigned char **blocks, t_ihv **ihvs, const char *fname);
void dump_blocks_ihvs(unsigned char *blocks, t_ihv *ihvs);
