/*
 * utils.c
 */

#include <stdlib.h>
#include <stdio.h>
#include <stdint.h>
#include "common.h"

uint32_t swapu32(uint32_t i)
{
    return (i << 24) | ((i << 8) & 0xff0000) |
	((i >> 8) & 0xff00) | ((i >> 24) & 0xff);
}

/*
 * Parses a node, eg. "x,y".
 */
void read_node(t_node *n, const char *s)
{
    if (2 != sscanf(s, "%x,%x", &n->x, &n->y))
	fprintf(stderr, "failed to read a node\n"), exit(1);
}

static void read_ihv(FILE *f, t_ihv *i)
{
    uint32_t a, b, c, d;
    if (4 != fscanf(f, "%08x %08x %08x %08x", &a, &b, &c, &d))
	fprintf(stderr, "failed to read 4 IHVs\n"), exit(1);
    i[0].a = swapu32(a);
    i[0].b = swapu32(b);
    i[0].c = swapu32(c);
    i[0].d = swapu32(d);
}

static void read_block(FILE *f, unsigned char *b)
{
    for (unsigned i = 0; i < MD5_BLK_BYTES; i++)
	if (1 != fscanf(f, "%02hhx", b + i))
	    fprintf(stderr, "failed to read a data byte\n"), exit(1);
}

/*
 * Reads 2 blocks and 2 IHVs from a file representing them in human-readable
 * format.
 */
void read_blocks_ihvs(unsigned char **blocks, t_ihv **ihvs, const char *fname)
{
    if (!(*blocks = malloc(2 * MD5_BLK_BYTES)))
	perror("malloc"), exit(1);
    if (!(*ihvs = malloc(2 * sizeof (**ihvs))))
	perror("malloc"), exit(1);
    FILE *f;
    if (!(f = fopen(fname, "r")))
	perror(fname), exit(1);
    read_ihv(f, *ihvs + 0);
    read_block(f, *blocks + 0);
    read_ihv(f, *ihvs + 1);
    read_block(f, *blocks + MD5_BLK_BYTES);
    if (fclose(f))
	perror("close"), exit(1);
}

/*
 * Dumps 2 blocks and 2 IHVs on stdout.
 */
void dump_blocks_ihvs(unsigned char *blocks, t_ihv *ihvs)
{
    for (unsigned j = 0; j < 2; j++)
      {
	printf("%08x %08x %08x %08x",
		swapu32(ihvs[j].a), swapu32(ihvs[j].b),
		swapu32(ihvs[j].c), swapu32(ihvs[j].d));
	for (unsigned i = 0; i < MD5_BLK_BYTES; i++)
	    printf("%s%02x", (!(i % 16) ? "\n" : (i % 16 == 8 ? "  " : " ")),
		    blocks[j * MD5_BLK_BYTES + i]);
	printf("\n");
      }
}
